package net.maunium.Maucros.Misc;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.NetworkInterface;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Pattern;

import net.maunium.Maucros.Maucros;

/**
 * A heavily modified version of MauLicencingSystem for Maucros.<br>
 * Uses it's own MaucrosLicensor server.
 * 
 * @author Tulir293
 * @since 1.4 Alpha 1
 */
public class MaucrosLicensor {
	private static boolean licenced = false;
	private static boolean ksd = true;
	private static Map<String, String> licences = new HashMap<String, String>();
	
	/**
	 * Check if the version of Maucros is killswitched.
	 */
	public static boolean isKillswitched() {
		return ksd;
	}
	
	/**
	 * Check if Maucros is licenced to run.
	 */
	public static boolean isLicenced() {
		return licenced;
	}
	
	/**
	 * Check if Maucros should be allowed to run.
	 */
	public static boolean check() {
		return licenced & !ksd;
	}
	
	/**
	 * Get all the MAC-Licence combinations in a hashmap.
	 */
	public static Map<String, String> getLicences() {
		return licences;
	}
	
	public static void addLicence(String mac, String licence) {
		licences.put(mac, licence);
	}
	
	/**
	 * Saves the licences to file.
	 * 
	 * @throws IOException If writing to the file fails
	 */
	public static void saveLicences() throws IOException {
		BufferedWriter bw = new BufferedWriter(new FileWriter(new File(System.getProperty("user.dir") + File.separator + "config" + File.separator + "Maucros"
				+ File.separator + "licences.maudat")));
		for (Entry<String, String> ent : MaucrosLicensor.getLicences().entrySet()) {
			bw.write(ent.getKey() + "~" + ent.getValue());
			bw.newLine();
		}
		bw.close();
	}
	
	/**
	 * Loads the licence(s) from the licence file.
	 * 
	 * @throws IOException If reading the file fails
	 */
	public static void loadLicences() throws IOException {
		// Define the file.
		File f = new File(System.getProperty("user.dir") + File.separator + "config" + File.separator + "Maucros" + File.separator + "licences.maudat");
		// If the folder doesn't exist, make it.
		if (!f.getParentFile().exists()) f.getParentFile().mkdirs();
		// Return if no licence file found.
		if (!f.exists()) return;
		
		// Create a buffered reader.
		BufferedReader br = new BufferedReader(new FileReader(f));
		
		// Read loop
		String s;
		while ((s = br.readLine()) != null)
			// Check if it's a valid mac~licence combination
			if (s.contains("~")) {
				// Valid combination, split it to MAC and Licence.
				String[] ss = s.split(Pattern.quote("~"));
				// Add the combination to te licence map
				licences.put(ss[0], ss[1]);
			}
		
		// All done, close the bufferedreader.
		br.close();
	}
	
	/**
	 * Queries the MaucrosLicensor server for killswitch status and licence validation
	 * 
	 * @throws UnknownHostException If maunium.net can't be resolved.
	 * @throws IOException If reading/writing data fails
	 */
	public static void query() throws UnknownHostException, IOException {
		String version = Maucros.ksv;
		Socket s;
		s = new Socket("maunium.net", 29352);
		OutputStream out = s.getOutputStream();
		InputStream in = s.getInputStream();
		byte[] b = version.getBytes(StandardCharsets.UTF_8);
		out.write(b.length);
		out.write(b);
		int response;
		response = in.read();
		if (response == 0) ksd = false;
		
		NetworkInterface network = null;
		for (NetworkInterface ni : Collections.list(NetworkInterface.getNetworkInterfaces())) {
			if (!ni.isUp() || ni.isVirtual() || ni.isLoopback() || ni.isPointToPoint()) continue;
			network = ni;
			break;
		}
		if (network == null) {
			out.write(125);
			out.write(125);
			s.close();
			throw new SocketException("No valid network interfaces found!");
		}
		byte[] mac = network.getHardwareAddress();
		
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < mac.length; i++)
			sb.append(String.format("%02X%s", mac[i], i < mac.length - 1 ? "-" : ""));
		String lmac = sb.toString();
		
		String licence = licences.get(lmac);
		if (licence == null) {
			out.write(125);
			out.write(125);
			s.close();
			return;
		}
		b = licence.getBytes(StandardCharsets.UTF_8);
		out.write(b.length);
		if (b.length == 125) out.write(1);
		out.write(b);
		b = lmac.getBytes(StandardCharsets.UTF_8);
		out.write(b.length);
		out.write(b);
		
		response = in.read();
		s.close();
		if (response == 0) licenced = true;
	}
}
