/*
 * ============================================================================= Copyright (c) 2007-2010, The JASYPT team (http://www.jasypt.org) Licensed under
 * the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License. =============================================================================
 */
package org.jasypt.misc;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import org.jasypt.exceptions.EncryptionInitializationException;

/**
 * <p>
 * Utility for the normalization of Unicode Strings to NFC form.
 * </p>
 * <p>
 * This class tries to use the <tt>java.text.Normalizer</tt> class in JDK 1.6 first and, if it the class is not found (Java version < 6), then it will use the
 * ICU4J <tt>com.ibm.icu.text.Normalizer</tt> class (in this case, a <tt>ClassNotFoundException</tt> will be thrown if ICU4J is not present).
 * </p>
 * 
 * @since 1.5
 * 
 * @author Daniel Fern&aacute;ndez
 * @author Tulir293
 */
public final class Normalizer {
	
	private static final String JDK_NORMALIZER_CLASS_NAME = "java.text.Normalizer";
	private static final String JDK_NORMALIZER_FORM_CLASS_NAME = "java.text.Normalizer$Form";
	
	private static boolean initialized = false;
	
	private static Method javaTextNormalizerMethod = null;
	private static Object javaTextNormalizerFormNFCConstant = null;
	
	/**
	 * <p>
	 * Normalize Unicode-input message to NFC.
	 * </p>
	 * <p>
	 * This algorithm will first try to normalize the input's UNICODE using icu4j's <tt>com.ibm.icu.text.Normalizer</tt> and, if it is not present at the
	 * classpath, will try to use <tt>java.text.Normalizer</tt>. If this is not present either (this class appeared in JavaSE 6), it will raise an exception.
	 * </p>
	 * 
	 * @param message the message to be normalized
	 * @return the result of the normalization operation
	 */
	public static String normalizeToNfc(final String message) {
		return new String(normalizeToNfc(message.toCharArray()));
	}
	
	/**
	 * <p>
	 * Normalize Unicode-input message to NFC.
	 * </p>
	 * <p>
	 * This algorithm will first try to normalize the input's UNICODE using icu4j's <tt>com.ibm.icu.text.Normalizer</tt> and, if it is not present at the
	 * classpath, will try to use <tt>java.text.Normalizer</tt>. If this is not present either (this class appeared in JavaSE 6), it will raise an exception.
	 * </p>
	 * 
	 * @param message the message to be normalized
	 * @return the result of the normalization operation
	 */
	public static char[] normalizeToNfc(final char[] message) {
		if (!initialized) try {
			initializeJavaTextNormalizer();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return normalizeWithJavaNormalizer(message);
		
	}
	
	static void initializeJavaTextNormalizer() throws ClassNotFoundException, NoSuchMethodException, NoSuchFieldException, IllegalAccessException {
		final Class<?> javaTextNormalizerClass = Thread.currentThread().getContextClassLoader().loadClass(JDK_NORMALIZER_CLASS_NAME);
		final Class<?> javaTextNormalizerFormClass = Thread.currentThread().getContextClassLoader().loadClass(JDK_NORMALIZER_FORM_CLASS_NAME);
		javaTextNormalizerMethod = javaTextNormalizerClass.getMethod("normalize", new Class[] { CharSequence.class, javaTextNormalizerFormClass });
		final Field javaTextNormalizerFormNFCConstantField = javaTextNormalizerFormClass.getField("NFC");
		javaTextNormalizerFormNFCConstant = javaTextNormalizerFormNFCConstantField.get(null);
		
		initialized = true;
	}
	
	static char[] normalizeWithJavaNormalizer(final char[] message) {
		
		if (javaTextNormalizerMethod == null || javaTextNormalizerFormNFCConstant == null) { throw new EncryptionInitializationException("Cannot use: "
				+ JDK_NORMALIZER_FORM_CLASS_NAME + ", as JDK-based normalization has " + "not been initialized! (check previous execution errors)"); }
		
		// Using java JDK's Normalizer, we cannot avoid creating Strings
		// (it is the only possible interface to the Normalizer class).
		//
		// Note java.text.Normalizer is accessed via reflection in order to allow this
		// class to be JDK 1.4-compilable (though ICU4j will be needed at runtime
		// if Java 1.4 is used).
		final String messageStr = new String(message);
		final String result;
		try {
			result = (String) javaTextNormalizerMethod.invoke(null, new Object[] { messageStr, javaTextNormalizerFormNFCConstant });
		} catch (final Exception e) {
			throw new EncryptionInitializationException("Could not perform a valid UNICODE normalization", e);
		}
		return result.toCharArray();
	}
	
	private Normalizer() {
		super();
	}
	
}
